/*
 *  Project   : SPECTRUM - Digital Spectrometric System of Nuclear Radiation
 *
 *  Tool      : ADC Data Acquisition to File
 *               
 *  Author(s) : Martin VESKRNA <256569@mail.muni.cz>             
 *
 *  Date      : 
 *
 *  (c) Copyright 2011 MASARYK UNIVERSITY
 *  All rights reserved.
 *
 *  Please review the terms of the license agreement before using this
 *  file. If you are not an authorized user, please destroy this
 *  source code file and notify MASARYK UNIVERSITY immediately that you
 *  inadvertently received an unauthorized copy.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <unistd.h>
#include <limits.h>
#include <time.h>

#include "UDPsending.h"

#define BUFLEN 2000
#define PORT 9930

#define PACKET_HEADER_LENGTH 18

#define MAX_PULSES_IN_PACKET 300

char buf[BUFLEN];
char* bufPointer;

struct sockaddr_in si_other;
int s, i, slen=sizeof(si_other);

unsigned char isFirstPulseInPacket;
unsigned short pulsesInPacket;

void ResetBuffer()
{
	bufPointer = buf + PACKET_HEADER_LENGTH;
	isFirstPulseInPacket = 1;
	pulsesInPacket = 0;
}

int PrepareUDP(char* IP)
{
	if ((s=socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP))==-1)
	{
		fprintf(stderr, "Can not create socket for UDP transferee\n");
		return 0;
	}

	memset((char *) &si_other, 0, sizeof(si_other));
	si_other.sin_family = AF_INET;
	si_other.sin_port = htons(PORT);
	
	if (inet_aton(IP, &si_other.sin_addr)==0)
	{
		fprintf(stderr, "inet_aton() failed\n");
		return 0;
	}
	
	ResetBuffer();
	
	UDPpacketNumber = 0;
	
	return 1;
}

void CloseUDP()
{
	close(s);
}


//TODO: SendViaUDT jen zape do bufferu a paket je posln
		// pokud je as dvou poslednch puls pli velk - aby byl penos co nejkontinulnj
		// asova poslajc iju pakety vyprzdn buffer (aby data nestrla) - jednou za 1ms se ukon blokovn szeRead a tm pdem se zavol posln przdnho UDP paketu == iju
		
		
		
time_t previousTimeUDP;
		
int SendPulseViaUDP(unsigned long long ComboTime, unsigned int pulseNum, unsigned long long valueA, unsigned long long valueB)
{
	//check if the incomming data is so high
	if (valueA > USHRT_MAX || valueB > USHRT_MAX)
	{
		fprintf(stderr, "valueA or valueB is grater then maximum\n");
		return 0;
	}
	//printf("a\n");
	//store data to buffer
	unsigned short A = valueA;
	unsigned short B = valueB;
	memcpy(bufPointer, (void *) &A, sizeof(A));
	memcpy(bufPointer+2, (void *) &B, sizeof(B));
	bufPointer += 4;
	pulsesInPacket++;
	
	//write UDP header - data from first pulse
	if (isFirstPulseInPacket == 1)
	{
		memcpy(buf+6, (void *) &pulseNum, sizeof(pulseNum));
		memcpy(buf+10, (void *) &ComboTime, sizeof(ComboTime));
		isFirstPulseInPacket = 0;
	}
	

	//output packet is sent if the size is maximum or every second
	time_t actualTime = time ( NULL );
	if (pulsesInPacket >= MAX_PULSES_IN_PACKET || actualTime != previousTimeUDP)
	{
		if (SendUDPmethodPacket() == 0)
		{
			return 0;
		}
	}
	previousTimeUDP = actualTime;
	
	return 1;
}


int SendUDPmethodPacket()
{
	//packet header
	memcpy(buf, (void *) &UDPpacketNumber, sizeof(UDPpacketNumber));
	memcpy(buf+4, (void *) &pulsesInPacket, sizeof(pulsesInPacket));
	
	if (SendUDPpacket(buf, bufPointer - buf) == 0)
	{
		return 0;
	}
	
	ResetBuffer();
	
	return 1;
}


int SendUDPpacket(char* buffer, unsigned int length)
{
	if (sendto(s, buffer, length, 0, (struct sockaddr*)&si_other, slen) == -1)
    {
		fprintf(stderr, "Error while UDP packet sending\n");
		return 0;
	}
	
	UDPpacketNumber++;
	
	return 1;
}

