/*
 *	 Project     : SPECTRUM - Digital Spectrometric System of Nuclear Radiation
 *
 *	 Tool        : ADC configuration file parser
 *
 *	 Author(s)   : Martin VESKRNA <256569@mail.muni.cz>
 *
 *	 Date        : $Id: configFile.c 142 2013-05-10 16:33:36Z 256357 $
 *
 *	 (c) Copyright 2011 MASARYK UNIVERSITY
 *	 All rights reserved.
 *
 *	 Please review the terms of the license agreement before using this
 *	 file. If you are not an authorized user, please destroy this
 *	 source code file and notify MASARYK UNIVERSITY immediately that you
 *	 inadvertently received an unauthorized copy.
 *
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "configFile.h"


//read one line in DAT file and return it in structure
int ReadDATline(FILE* file, t_DATline *line)
{		
	int c = fgetc(file);
	
	if (c == EOF)
	{
		line->type = LINE_EOF;
	}
	else if (isspace(c))
	{
		//case if at line is any white spaces (include empty line)
		while (c != '\n')
		{
			c = fgetc(file);			
		}
		line->type = LINE_EMPTY;
	}
	else if (c == '#')
	{
		if (fscanf(file, "%50s", line->paramName) <= 0)	
		{
			return 0;
		}
		
		//read spaces
		do 
		{
			c = fgetc(file);			
		} while (c == ' ' || c == '\t');
		ungetc(c, file);
		
		//detect if there is no parameters
		if (c == '\n' || c == EOF)
		{
			 line->paramValue[0] = '\0';
		}
		else
		{
			if (fgets(line->paramValue, 100, file) == NULL)	
			{
				return 0;
			}
		}
		
		//remove new line character at the end of this line and check that we are at the end of line
		int len = strlen(line->paramValue)-1;
		if (line->paramValue[len] != '\n')
		{
			//we did not find \n, probably the line is longer than 100 characters
			do 
			{
				c = fgetc(file);			
			} while (c != '\n' && c != EOF);
		}
		
		//remove \n and white spaces at the end of line
		while (len >= 0 && isspace(line->paramValue[len]))
		{
			line->paramValue[len] = '\0';
			len--;
		}
		
		line->type = LINE_PARAMETER;
	}
	else if (isdigit(c))
	{
		ungetc(c, file);
		if (fscanf(file, "%hu", &(line->number)) <= 0)
		{
			return 0;
		}
		
		//read all rest character at this line
		do 
		{
			c = fgetc(file);			
		} while (c != '\n' && c != EOF);
		
		line->type = LINE_NUMBER; 
	}
	else
	{
		return 0;
	}
	
	return 1;
}
