/*
 *	 Project     : SPECTRUM - Digital Spectrometric System of Nuclear Radiation
 *
 *	 Tool        : ADC configuration file parser header file
 *
 *	 Author(s)   : Martin VESKRNA <256569@mail.muni.cz>
 *
 *	 Date        : $Id: configFile.h 144 2013-05-23 15:24:03Z 256357 $
 *
 *	 (c) Copyright 2011 MASARYK UNIVERSITY
 *	 All rights reserved.
 *
 *	 Please review the terms of the license agreement before using this
 *	 file. If you are not an authorized user, please destroy this
 *	 source code file and notify MASARYK UNIVERSITY immediately that you
 *	 inadvertently received an unauthorized copy.
 *
 */
 
#ifndef __CONFIGFILE_H_
#define __CONFIGFILE_H_
#include <stdio.h>

#define LINE_EOF		-1
#define LINE_EMPTY		 0
#define LINE_NUMBER		 1
#define LINE_PARAMETER	 2

/**
 * Data structure for storing information about 1 line from config file.
 */
typedef struct s_DATline
{
	unsigned int type; 		//one of defines above
	char paramName[50]; 	//name of the parameter (value after #)
	char paramValue[100];   //value of the parameter (value after space)
	unsigned short number;
} t_DATline;


/**
 * read one line in DAT file and return it in structure
 * @param file opened config file
 * @param line will be filled with parsed line of that file
 * @return 0 when some invalid character is present on the line or on some other error,
 * 			 values of line is undefined
 * 		   1 when data line was read, in this case line will be filled with values
 */
int ReadDATline(FILE* file, t_DATline *line);

#endif /* CONFIGFILE_H_ */
