/*
 *  Project   : SPECTRUM - Digital Spectrometric System of Nuclear Radiation
 *
 *  Tool      : ADC measuring procedures source file
 *
 *  Author(s) : Martin CECH <256357@mail.muni.cz>
 *
 *  Date      : $Id: measuring.c 332 2014-03-13 15:12:37Z 256569 $
 *
 *  (c) Copyright 2011 MASARYK UNIVERSITY
 *  All rights reserved.
 *
 *  Please review the terms of the license agreement before using this
 *  file. If you are not an authorized user, please destroy this
 *  source code file and notify MASARYK UNIVERSITY immediately that you
 *  inadvertently received an unauthorized copy.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>

#include "standalone.h"
#include "measuring.h"

#define PACKET_HEADER_LENGTH (6+6+2+4+8+4+2)
#define PACKET_FOOTER_LENGTH (1)
#define PACKET_MAX_LENGTH 2000

#define BUFFER_LENGTH_READING 3000000000llu
#define BUFFER_LENGTH_TESTING 100000000llu
#define BUFFER_LENGTH_THRESHOLD_ASSISTANT 200000000llu

#define STANDALONE_TIMEOUT_MS 300
#define STANDALONE_RETRANSMIT 5

int OpenInterface(tMeasureParams *measureParams)
{
	if (measureParams->standalone){
		printf("Opening pcap interface(%s)\t", measureParams->standaloneEthName);
		fflush(stdout);

		if (openPcapDev(measureParams->standaloneEthName) != 0){
			return 0;
		}else{
			return 1;
		}
	}else{
		
	}
}

void CloseInterface(tMeasureParams *measureParams)
{
	if (measureParams->standalone){
		closePcapDev();
	}else{
		
	}
}

int ClearInterface(tMeasureParams *measureParams)
{
	if (measureParams->standalone){
		clearPcapBuffer();
	}else{
		
	}
	return 1;
}

int PingInterface(tMeasureParams *p)
{
	int ret;
	if (p->standalone){
		if((ret = sendPacketWaitAck(PING_PACK, p, STANDALONE_TIMEOUT_MS, STANDALONE_RETRANSMIT))!=2) {
			return 0;
		}
	}else{
		//TODO ping na combo kart
	}
	return 1;
}
int ADCs_Calibrate(tMeasureParams *p, int runCalib)
{
	//FPGA CONFIG registr
	p->ADC12_oen = 0;
	p->ADC12_calib = runCalib;

	int ret;
	if (p->standalone){
		//fpga config register
		if((ret = sendPacketWaitAck(FPGA_CONF_PACK, p, STANDALONE_TIMEOUT_MS, STANDALONE_RETRANSMIT))!=2) {
			return 0;
		}
		//ADC1 config register
		if (p->ADC1_on){
			if(sendPacketWaitAck(ADC1_PACK, p, STANDALONE_TIMEOUT_MS, STANDALONE_RETRANSMIT)!=2) {
				return 0;
			}
		}

		//ADC2 config register
		if (p->ADC2_on){
			if(sendPacketWaitAck(ADC2_PACK, p, STANDALONE_TIMEOUT_MS, STANDALONE_RETRANSMIT)!=2) {
				return 0;
			}
		}
	}else{
		
	}
	return 1;
}

int ADCs_LoadParams(tMeasureParams *p)
{
	p->ADC12_oen = 0;
	p->ADC12_calib = 0;

	if (p->standalone){
		if (sendPacketWaitAck(FPGA_CONF_PACK, p, STANDALONE_TIMEOUT_MS, STANDALONE_RETRANSMIT) != 2){
			return 0;
		}
	}else{
		
	}

	if (p->ADC1_on){
		if (p->standalone){
			if (sendPacketWaitAck(ADC1_PACK, p, STANDALONE_TIMEOUT_MS, STANDALONE_RETRANSMIT) != 2){
				return 0;
			}
		}else{
			
		}
	}

	if (p->ADC2_on){

		if (p->standalone){
			if (sendPacketWaitAck(ADC2_PACK, p, STANDALONE_TIMEOUT_MS, STANDALONE_RETRANSMIT) != 2){
				return 0;
			}
		}else{
			
		}
	}

	return 1;
}

int ADCs_setThreshold(tMeasureParams *p)
{
	if (p->ADC1_on){
		if (p->standalone) {
			if (sendPacketWaitAck(ADC1_PACK, p, STANDALONE_TIMEOUT_MS, STANDALONE_RETRANSMIT) != 2) return 0;
		}
		else {
			
		}
	}

	if (p->ADC2_on){
		if (p->standalone){
			if (sendPacketWaitAck(ADC2_PACK, p, STANDALONE_TIMEOUT_MS, STANDALONE_RETRANSMIT) != 2) return 0;
		}
		else {
			
		}
	}
	return 1;
}


int MeasureStart(tMeasureParams *p)
{

	p->ADC12_oen = 1;

	if (p->standalone){
		if (sendPacketWaitAck(FPGA_CONF_PACK, p, STANDALONE_TIMEOUT_MS, STANDALONE_RETRANSMIT) != 2){
			return 0;
		}
	}else{
		
	}
	return 1;
}

int MeasureStop(tMeasureParams *p)
{
	p->ADC12_oen = 0;
	int adc1on = p->ADC1_on;
	int adc2on = p->ADC2_on;
	p->ADC1_on = 0;
	p->ADC2_on = 0;
	if (p->standalone){
		if (sendPacketWaitAck(FPGA_CONF_PACK, p, STANDALONE_TIMEOUT_MS+200,STANDALONE_RETRANSMIT) != 2){
			p->ADC1_on = adc1on;
			p->ADC2_on = adc2on;
			return 0;
		}
	}else{
		
	}
	p->ADC1_on = adc1on;
	p->ADC2_on = adc2on;
	return 1;
}
