/*
 *  Project   : SPECTRUM - Digital Spectrometric System of Nuclear Radiation
 *
 *  Tool      : ADC measuring procedures header file
 *
 *  Author(s) : Martin CECH <256357@mail.muni.cz>
 *
 *  Date      : $Id: measuring.h 216 2013-11-25 22:32:02Z 256569 $
 *
 *  (c) Copyright 2011 MASARYK UNIVERSITY
 *  All rights reserved.
 *
 *  Please review the terms of the license agreement before using this
 *  file. If you are not an authorized user, please destroy this
 *  source code file and notify MASARYK UNIVERSITY immediately that you
 *  inadvertently received an unauthorized copy.
 *
 */

#ifndef __MEASURING_H_
#define __MEASURING_H_
#include "parameters.h"

#define N_OF_ADCS 2

/*
 * Opens interface for communication with ADC card - pcap or sze
 * @param measureParams structure with parameters as readed from config file
 */
int OpenInterface(tMeasureParams *measureParams);
/*
 * Closes interface previously opened for communictaion with ADC card - pcap or sze
 * @param measureParams structure with parameters as readed from config file
 */
void CloseInterface(tMeasureParams *measureParams);
/*
 * Reads all buffered packet in previously opened interface and throws them.
 * @param measureParams structure with parameters as readed from config file
 * @return 1 when clear OK, else 0
 */
int ClearInterface(tMeasureParams *measureParams);

/*
 * Try to communicate witch ADC card.
 * @param measureParams structure with parameters as readed from config file
 * @return 1 when ping OK, else 0
 */
int PingInterface(tMeasureParams *measureParams);
/*
 * Sends calibration message to ADC card through previously opened interface.
 * @param measureParams structure with parameters as readed from config file
 */
int ADCs_Calibrate(tMeasureParams *p, int runCalib);
/*
 * Loads measuring parameters into ADC card through previously opened interface.
 * @param measureParams structure with parameters as readed from config file
 */
int ADCs_LoadParams(tMeasureParams *p);
/*
 * Sends threshold parameter to ADC card through previously opened interface.
 * Threshold is send by function ADCs_LoadParams as well, but this function
 * is usefull when only threshold should be set.
 * @param measureParams structure with parameters as readed from config file
 */
int ADCs_setThreshold(tMeasureParams *p);

/*
 * Starts meassuring with ADC card. Interface must be opened and properly set (through
 * functions above). This procedure starts children processes for buffering,
 * HDD saving etc.
 * @param measureParams structure with parameters as readed from config file
 * @param filePrefix is prefix to output files to which will be writen the output.
 */
int MeasureStart(tMeasureParams *p);
/*
 * Stops currently running measurement by sending "stop" messages to all the children processes.
 * @param measureParams structure with parameters as readed from config file
 */
int MeasureStop(tMeasureParams *p);

#endif /* MEASURING_H_ */
