/*
 *  Project   : SPECTRUM - Digital Spectrometric System of Nuclear Radiation
 *
 *  Tool      : ADC card FPGA flashing program
 *
 *  Author(s) : Martin CECH <256357@mail.muni.cz>
 *
 *  Date      : $Id: spektromeasure.c 142 2013-05-10 16:33:36Z 256357 $
 *
 *  (c) Copyright 2011 MASARYK UNIVERSITY
 *  All rights reserved.
 *
 *  Please review the terms of the license agreement before using this
 *  file. If you are not an authorized user, please destroy this
 *  source code file and notify MASARYK UNIVERSITY immediately that you
 *  inadvertently received an unauthorized copy.
 *
 */

#include <stdlib.h>
#include <stdio.h>
#include <pcap.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <signal.h>
#include "standalone.h"

#define EXP_FILE_SIZE 3280023
int runProgram = 1;

void signal_handler(int signum)
{
	printf("\nFlashing not complete, device will not boot properly after reset!\n"
			"Abort flashing [y/n]?");
	char ans[] = "n";
	ans[0]=(char)getchar();
	fflush(stdin);
	if (strcasecmp("y", ans) == 0){
		runProgram = 0;
	}else{
		runProgram = 1;
	}
}

void PrintHelp()
{
	printf("Usage: spektroflash <interface> <path_to_bitsream>\n");
	printf("       spektroflash <interface> -[rcpl]\n");
	printf("Flash FPGA with new design \n"
			"or restart device (-r) \n"
			"or erase flash chip (-c) \n"
			"or ping device (-p).\n"
			"or send pll packet (-l).\n");
}

int main(int argc, char **argv)
{
	int ret;
	tMeasureParams pars;
	MeasureParamsInit(&pars);
	//print packet assumptions

	if (argc != 3)
	{
		PrintHelp();
		return 0;
	}

	strcpy(pars.standaloneEthName, argv[1]);
	strcpy(pars.bitfile, argv[2]);
	if (openPcapDev(argv[1]) != 0){
		return -1;
	}
	int i = 0;

	if (strcmp(argv[2], "-r") == 0){
		printf("Restarting device.\n");
		ret = sendPacketWaitAck(RESET_PACK, &pars, 15000, 0);
		if (ret == 2) {printf(" Device restarted.\n");}
		if (ret == 1) {printf(" Wrongly acked data!\n");}
		if (ret == 0) {printf(" Not acked data!\n");}
		if (ret == -1){printf(" Communication error!\n");}
		return 0;
	}
	if (strcmp(argv[2], "-c") == 0){
		printf("Erasing flash chip.\n");
		ret = sendPacketWaitAck(FLASH_ERASE_PACK, &pars, 20000, 0);
		if (ret == 2) {printf(" Chip erased.\n");}
		if (ret == 1) {printf(" Wrongly acked data!\n");}
		if (ret == 0) {printf(" Not acked data!\n");}
		if (ret == -1){printf(" Communication error!\n");}
		return 0;
	}
	if (strcmp(argv[2], "-p") == 0){
		printf("Sending ping.\n");
		ret = sendPacketWaitAck(PING_PACK, &pars, 1000, 2);
		if (ret == 2) {printf(" Ack received.\n");}
		if (ret == 1) {printf(" Wrongly acked data!\n");}
		if (ret == 0) {printf(" Not acked data!\n");}
		if (ret == -1){printf(" Communication error!\n");}
		return 0;
	}
	if (strcmp(argv[2], "-l") == 0){
		printf("Sending pll packet.\n");
		ret = sendPacketWaitAck(PLL_PACK, &pars, 2000, 0);
		if (ret == 2) {printf(" Ack received.\n");}
		if (ret == 1) {printf(" Wrongly acked data!\n");}
		if (ret == 0) {printf(" Not acked data!\n");}
		if (ret == -1){printf(" Communication error!\n");}
		return 0;
	}
	struct stat bitfileStat;
	printf("Checking input file");
	if (stat(argv[2], &bitfileStat)!=0){
		perror("Problem with getting status of bit file");
	}

	if (bitfileStat.st_size == EXP_FILE_SIZE){
		printf("\tOK\n");fflush(stdout);
	}else{
		printf("\tFALSE\n Size of file: \t\t%u\n Expected size: \t%u\n", (int)bitfileStat.st_size, EXP_FILE_SIZE);fflush(stdout);
		return -1;
	}
	printf("Sending packet data...\n");fflush(stdout);
	char ans[] = "n";

	signal(SIGINT, signal_handler);
	while (runProgram){
		ret = sendPacketWaitAck(BITSTREAM_PACK, &pars, 500,5);
		if (ret == 3) {
			printf("\nSending of bitstream file succeeded!\nReset device[y/n]?");
			ans[0] = (char)getchar();
			fflush(stdin);
			break;
		}
		if (ret == 2) {i+=256;printf("\r\t%dB of data send (%d packets).         %.2f%%   ",i, i/256, (i*100)/(float)EXP_FILE_SIZE);fflush(stdout);continue;}
		if (ret == 1) {printf("\nWrongly acked data!\n");fflush(stdout);break;}
		if (ret == 0) {printf("\nNot acked data!\n");fflush(stdout);break;}
		if (ret == -1){printf("\nCommunication error!\n");fflush(stdout);break;}
		if (ret == -2) {printf("\nCan't open file '%s'!\n", argv[2]);fflush(stdout);break;}
	}
	signal(SIGINT, SIG_DFL);
	if (runProgram && (strcasecmp("y", ans) == 0)){
		printf("Restarting device.\n");
		ret = sendPacketWaitAck(RESET_PACK, &pars, 15000, 2);
		if (ret == 2) {printf("\nDevice restarted.\n");}
		if (ret == 1) {printf("\nWrongly acked data!\n");}
		if (ret == 0) {printf("\nNot acked data!\n");}
		if (ret == -1){printf("\nCommunication error!\n");}
	}else{
		printf("\nDevice not restarted.\n");
	}
	fflush(stdout);
	closePcapDev();
	return 0;
}	
