/*
 *  Project   : SPECTRUM - Digital Spectrometric System of Nuclear Radiation
 *
 *  Tool      : ADC program for ADC card control and data acquisition header file
 *
 *  Author(s) : Martin VESKRNA <256569@mail.muni.cz> 
 *              Martin CECH <256357@mail.muni.cz> 
 *
 *  Date      : $Id: spektromeasure.h 326 2014-03-11 10:35:03Z 256569 $
 *
 *  (c) Copyright 2011 MASARYK UNIVERSITY
 *  All rights reserved.
 *
 *  Please review the terms of the license agreement before using this
 *  file. If you are not an authorized user, please destroy this
 *  source code file and notify MASARYK UNIVERSITY immediately that you
 *  inadvertently received an unauthorized copy.
 *
 */

#ifndef __SPEKTROMEASURE_H_
#define __SPEKTROMEASURE_H_
#include <time.h>
#include "testing.h"
#include "storingData.h"

//length of ethernet header
#define PACKET_HEADER_ETHERNET_LENGTH (6+6+2)
//length of oscilo packet header
#define PACKET_HEADER_LENGTH (PACKET_HEADER_ETHERNET_LENGTH+1+4+8+4+1)
//length of oscilo packet footer
#define PACKET_FOOTER_LENGTH (1)
//maximum packet length
#define PACKET_MAX_LENGTH 2000
//length of buffers
#define BUFFER_LENGTH_READING 10000000000llu
#define BUFFER_LENGTH_TESTING 100000000llu

#define ADC1_ID 0
#define ADC2_ID 1

typedef struct sPacketDecodingInfo
{
	unsigned long long prevPacketTimeEnd_HWticks;
	unsigned int prevDataStamp;
	unsigned int prevPacketStamp;

	int firstRun;

	//for writing numer of cached packets
	time_t previousTime;

	char ID;

	tTestParams testParams;
	tHDDparams HDDparams;

	//statistics
	unsigned long long catchedPackets;
	unsigned long long lostPackets;
	unsigned long long catchedPacketsPrevious;
	unsigned long long lostPacketsPrevious;

} tPacketDecodingInfo;

typedef struct sBuffer
{
	char* buffer;
	char* bufPointerWrite;
	char* bufPointerRead;
	char* bufStop;

	unsigned long long bufferedPackets[2];
	unsigned long long previousBufferedPackets[2];

} tBuffer;



#endif /* SPEKTROMEASURE_H_ */
