/*
 *  Project   : SPECTRUM - Digital Spectrometric System of Nuclear Radiation
 *
 *  Tool      : Communication with ADC card through ethernet interface 
 *              and pcap header file
 *
 *  Author(s) : Martin CECH <256357@mail.muni.cz>               
 *
 *  Date      : $Id: standalone.h 144 2013-05-23 15:24:03Z 256357 $
 *
 *  (c) Copyright 2011 MASARYK UNIVERSITY
 *  All rights reserved.
 *
 *  Please review the terms of the license agreement before using this
 *  file. If you are not an authorized user, please destroy this
 *  source code file and notify MASARYK UNIVERSITY immediately that you
 *  inadvertently received an unauthorized copy.
 *
 */
 
#ifndef __STANDALONE_H__
#define __STANDALONE_H__
#include "parameters.h"

#define HEADER_LEN 14
#define CONFIG_PACKET_LEN 1+55

#define BITSTREAM_DATA_LEN 256
#define BITSTREAM_PACKET_LEN 1+3+BITSTREAM_DATA_LEN

/**
 * Types of packets
 */
typedef enum{
	PLL_PACK=0,
	ADC1_PACK,
	ADC2_PACK,

	FPGA_CONF_PACK = 0x10,
	BITSTREAM_PACK = 0x20,
	RESET_PACK = 0x30,
	FLASH_ERASE_PACK = 0x40,
	PING_PACK = 0x7F
} t_packetType;

/**
 * Opens pcap ethernet session.
 * @param dev name of ethernet interface
 *        when NULL pcap will lookup for avaible interfaces, prints them on stdout
 *        and then asks user to select one option on stdin
 * @return 0 when correctly connected
 *         1 when some problem occured during creating session
 */
int openPcapDev(char *dev);

/**
 * Close pcap ethernet session and cleans up.
 */
void closePcapDev();


/**
 * Reads next waiting packet from previously opened pcap session.
 * @param length returns the length of received packet or undefined when no data received
 * @return pointer to data packet or NULL, when no packet arrived
 */
unsigned char *readPcapNext(int *length);

/**
 * Clears all pending packets from interface.
 */
void clearPcapBuffer();

/**
 * Sends configuration packet through ethernet interface. Session must be previously opened by calling openDev function
 * @param type is type of packet
 * @param is tMeasureParams with parameters, some of them will be send according to selected type of packet
 * @return 3 when type==BITSTREAM_PACK and all data from bistream file allready send
 * 		   2 on successfully send and acked data
 *         1 on successfully send data but wrongly acked
 *         0 on successfully send data but not acked at all
 *         -1 when some error occured
 *         -2 when type==BITSTREAM_PACK and path to bitsream is invalid
 *
 */
int sendPacketWaitAck(t_packetType type, tMeasureParams *params, int timeoutms, int resendAttempts);

#endif 
