/*
 *  Project   : SPECTRUM - Digital Spectrometric System of Nuclear Radiation
 *
 *  Tool      : ADC testing modes
 *               
 *  Author(s) : Martin VESKRNA <256569@mail.muni.cz>             
 *
 *  Date      : $Id: testing.c 142 2013-05-10 16:33:36Z 256357 $
 *
 *  (c) Copyright 2011 MASARYK UNIVERSITY
 *  All rights reserved.
 *
 *  Please review the terms of the license agreement before using this
 *  file. If you are not an authorized user, please destroy this
 *  source code file and notify MASARYK UNIVERSITY immediately that you
 *  inadvertently received an unauthorized copy.
 *
 */
 
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "testing.h"
#include "parameters.h"
 ////////////////////////
 //  TEST PATTERN MODE //
/////////////////////////
void TestParamsInit(tTestParams *p)
{	
	p->testPatter_pointer = 0;
	p->testPatter_time = 0;
	p->testPatter_checkedPackets = 0;	
}

/*
T0 000h 004h 008h 010h
T1 FFFh FFBh FF7h FEFh
T2 000h 004h 008h 010h
T3 FFFh FFBh FF7h FEFh
T4 000h 004h 008h 010h
*/

static unsigned char testPatter_len = 6*5;
static char testPattern[] = {	0x00, 0x00, 0x04, 0x00, 0x80, 0x10,		//index  0.. 5
						0xFF, 0xFF, 0xFB, 0xFF, 0x7F, 0xEF,		//index  6..11
						0x00, 0x00, 0x04, 0x00, 0x80, 0x10,		//index 12..17
						0xFF, 0xFF, 0xFB, 0xFF, 0x7F, 0xEF,		//index 18..23
						0x00, 0x00, 0x04, 0x00, 0x80, 0x10		//index 24..29
						};
/*
 * function is simmilar to PLL, return number of additional pointer increment
*/
int CheckTestPattern(char* data, unsigned int dataLen, unsigned long long time_HWticks, tTestParams *p)
{		
	//arrival time of this sample
	unsigned long long timeAriv = ((time_HWticks) * ((HW_SAMPLES_IN_1_PERIOD * 12) / 8));
	
	unsigned long long timeAriv_atStart = timeAriv;
	
	p->testPatter_pointer = (unsigned char)(((unsigned long long)p->testPatter_pointer + (timeAriv - p->testPatter_time)) % testPatter_len);
	p->testPatter_time = timeAriv;
	
	unsigned char testPatter_pointer_atStart = p->testPatter_pointer;
	
	int error = 0;
	int errI;
	int errPatterI;
	
	int additionalInc = 0;
	int i = 0;
	while (i < dataLen)
	{
		int bytesToCompare = (testPatter_len - p->testPatter_pointer < dataLen - i) ? testPatter_len - p->testPatter_pointer : dataLen - i;
		
		if (memcmp(testPattern + p->testPatter_pointer, data + i, bytesToCompare) != 0)
		{
			if (!error)
			{
				error = 1;
				errI = i;
				errPatterI = p->testPatter_pointer;
			}
			
			p->testPatter_pointer++;
			//p->testPatter_time++;
			additionalInc++;
		}

		p->testPatter_pointer += (unsigned char)bytesToCompare;
		p->testPatter_pointer %= testPatter_len;
		
		p->testPatter_time += bytesToCompare;
		i += bytesToCompare;
	}
	
	p->testPatter_checkedPackets++;
	
	if (error && p->testPatter_checkedPackets > 300)
	{
		fprintf(stderr, "\nERROR at time %10llu    shift %4d      timeStart %10llu      pointerStart %4u     timeEnd %10llu    pointerEnd  %4u    \n", time_HWticks, additionalInc, timeAriv_atStart, testPatter_pointer_atStart, p->testPatter_time, p->testPatter_pointer);
		fprintf(stderr, "Expected data: ");
		
		//int toWrite = 30 < dataLen ? 30 : dataLen;
		int toWrite = dataLen;
		
		errPatterI -= errI;
		while (errPatterI < 0)
			errPatterI += testPatter_len;
			
		int i;
		for (i = 0; i < toWrite; i++)
		{
			fprintf(stderr, "%.2x ", (unsigned char)testPattern[errPatterI]);
			errPatterI++;
			errPatterI %= testPatter_len;
		}
		
		fprintf(stderr, "\nArrived data:  ");
		for (i = 0; i < toWrite; i++)
		{
			fprintf(stderr, "%.2x ", (unsigned char)data[i]);
		}
		
		fprintf(stderr, "\n");
	}
	
	return additionalInc;
}

//////////////////////////////////////////////////////////////////////////////////////////////////


////////////////////////
//  TIME TEST MODE    //
////////////////////////
static unsigned int packetsInSequence;
static unsigned int sequenceLengthPrepared = 0;
static unsigned int sequenceLength = 0;
static t_packetAssumption* packetAssumptions;

static unsigned char spaceLen[IMPULSES_IN_SEQUENCE] = {8,7,11,8,10,10,7,10,10,10,7,8,11,10,7,7,75,56,91,98,44,118,112,22,125,7,43,53,29,58,124,68,16,33,48,52,60,42,76,97,12,81,29,14,115,108,13,108,86,98,66,56,122,126,9,93,111,32,7,85,38,77,62,96,77,56,22,105,40,104,77,7,12,9,12,8,8,10,8,6,9,10,6,11,10,8,7,10,9,6,21,87,27,119,55,78,30,28,44,78,68,117,83,19,120,71,76,28,103,43,52,108,47,6,74,57,48,50,51,47,77,89,72,128,69,113,98,64,128,81,122,101,29,79,123,127,69,106,66,60,126,98,16,19,76,47,116,32,7,91,9,60,97,62,43,93,42,87,20,106,122,9,38,122,106,88,61,55,45,121,49,61,24,74,111,106,86,90,63,66,100,17,59,67,67,15,85,58,122,10,9,8,7,6,6,6,6,6,6,7,8,9,10,11,12,13,14,15,16,17,18,19,39,98,15,120,14,83,34,42,91,89,121,107,95,123,17,119,18,102,120,117,45,93,22,33,7,63,20,41,66,120,40,41,65,78,96,93,83,78,76,235,8,214,63,110,76,65,199,86,145,159,119,151,203,66,146,100,179,170,238,115,245,175,244,141,219,56,84,255,229,212,72,103,192,204,237,100,249,82,113,201,223,193,111,208,235,213,186,165,43,57,158,55,252,149,167,94,186,210,99,98,91,52,232,213,209,25,182,233,125,112,137,42,197,69,231,165,208,106,27,212,52,245,220,183,102,119,28,197,58,88,128,88,185,174,116,195,231,221,78,32,189,8,23,169,195,217,255,11,221,13,173,46,101,110,107,14,66,85,144,62,60,75,76,78,18,117,167,29,126,251,112,251,247,162,154,247,41,229,214,211,191,246,9,73,30,195,9,37,235,83,215,153,114,223,104,144,169,210,212,50,207,237,228,118,176,8,45,40,252,114,149,205,106,150,51,54,126,202,124,220,21,192,73,35,84,246,130,93,35,70,211,191,234,217,129,87,174,17,96,163,43,42,109,163,24,115,201,99,119,251,160,6,156,25,184,126,32,23,83,206,85,105,6,239,160,140,25,92,8,198,88,86,72,99,148,229,75,66,243,169,187,124,80,65,194,238,144,194,215,95,147,239,108,10,71,81,134,122,135,39};
							
static unsigned char pulseLen[IMPULSES_IN_SEQUENCE] = {13,9,13,26,26,32,29,24,16,2,10,24,6,16,17,21,42,47,34,50,30,28,61,9,44,26,54,35,26,14,22,126,25,4,60,94,51,105,50,61,57,65,15,52,4,120,85,91,130,187,117,61,21,74,30,202,68,241,94,246,78,37,194,231,223,18,183,236,196,152,76,93,249,69,214,189,206,255,140,33,116,146,144,30,166,176,124,80,219,165,241,194,158,84,61,45,112,177,139,113,250,90,189,60,203,120,102,44,111,143,242,125,45,110,144,124,48,209,194,207,96,255,83,226,179,228,125,106,72,245,255,98,4,104,2,118,7,250,100,46,187,235,126,211,198,66,214,118,159,35,209,239,35,71,237,85,214,63,139,195,52,184,203,221,191,92,140,244,49,122,16,67,113,31,75,13,193,202,110,241,242,243,244,245,246,247,248,249,250,251,252,253,254,255,255,255,254,253,252,251,250,249,248,247,246,245,244,243,242,241,240,30,31,32,33,34,35,36,35,34,33,32,31,30,29,28,45,203,98,174,217,224,137,213,32,55,17,151,233,104,153,74,11,250,25,247,41,58,103,82,245,52,126,156,156,8,142,7,204,114,149,137,68,187,159,197,79,238,84,249,18,132,2,131,25,101,198,7,9,121,252,185,214,143,165,224,124,233,216,250,136,56,11,182,106,63,39,34,247,251,71,52,249,138,35,36,65,154,215,162,23,173,63,126,127,247,119,171,47,32,191,77,155,172,87,182,18,93,134,246,56,71,115,238,232,233,245,128,192,28,7,223,89,253,247,127,145,201,32,184,189,205,30,164,73,70,88,47,133,114,236,245,54,159,166,248,172,118,58,152,222,84,84,246,98,248,185,68,181,72,20,238,188,121,154,85,220,97,53,237,153,65,81,124,29,191,249,76,156,109,121,16,236,149,12,76,73,149,223,251,157,160,15,88,68,21,161,87,169,69,91,234,171,244,114,140,36,69,205,10,149,116,5,52,64,24,189,105,174,52,141,127,2,229,20,43,247,249,102,76,12,32,127,68,158,31,214,210,135,185,199,121,253,151,244,251,57,255,68,10,187,154,172,78,165,21,112,195,2,232,231,126,101,8,160,65,223,188,205,163,141,140,217,12,196,77,41,5,62,86,192,236,31,247,240,142,81,17,181,165,20,11};

void PacketAssumptionsPrepare(unsigned int offsetPrev, unsigned int offsetPost)
{
	//compute sequence length
	sequenceLengthPrepared = 0;
	int i;
	for (i = 0; i < IMPULSES_IN_SEQUENCE; i++)
	{
		sequenceLengthPrepared += pulseLen[i] + spaceLen[i];
	}
	
	//allocate memory for packetAssumptions
	packetAssumptions = malloc(sequenceLengthPrepared * sizeof(t_packetAssumption));
	if (packetAssumptions == NULL)
	{
		fprintf(stderr, "Can not allocate memory for packetAssumptions.\n");
		exit(-1);
	}
	//fprintf(stderr, "A, %d\n", sequenceLengthPrepared);
	
	unsigned int time = 0;
	unsigned int packetNumber = 0;
	
	int impuls;
	for (impuls = 0; impuls < IMPULSES_IN_SEQUENCE; impuls++)
	{
		
		unsigned int impulsLenWithOffsets = pulseLen[impuls] + offsetPrev + offsetPost;
		
		//fprintf(stderr, "C, %d, %d\n", impulsLenWithOffsets, spaceLen[impuls] - offsetPrev - offsetPost);
		
		//add space whitch is not in any impuls
		int space;
		for (space = 0; space < spaceLen[impuls] - offsetPrev - offsetPost; space++)
		{
			packetAssumptions[time].isOffsetPrevPost = 0;
			packetAssumptions[time].canStartPacket = 0;
			
			time++;
		}
		//fprintf(stderr, "D\n");
		//own impuls - with offset prev and post
		int sample;
		for (sample = 0; sample < impulsLenWithOffsets; sample++)
		{
			//fprintf(stderr, "E\n");
			//this is the time when impuls start
			if (sample == 0)
			{
				packetAssumptions[time].canStartPacket = 1;
				
				packetAssumptions[time].FP_LP = 0xF0;	//this is first packet
				packetAssumptions[time].FP_LP |= (SAMPLES_IN_PACKET >= impulsLenWithOffsets) ? 0x0F : 0x00;
				
				packetAssumptions[time].packetLength = (SAMPLES_IN_PACKET < impulsLenWithOffsets) ? SAMPLES_IN_PACKET : impulsLenWithOffsets;
				packetAssumptions[time].packetNumber = packetNumber++;
				packetAssumptions[time].impulsNumber = impuls;
			}
			
			//time when start other packets
			else if (sample % SAMPLES_IN_PACKET == 0)
			{
				packetAssumptions[time].canStartPacket = 1;
				
				packetAssumptions[time].FP_LP = 0x00;	//this is first packet
				packetAssumptions[time].FP_LP |= (SAMPLES_IN_PACKET >= (impulsLenWithOffsets - sample)) ? 0x0F : 0x00;
				
				packetAssumptions[time].packetLength = (SAMPLES_IN_PACKET < (impulsLenWithOffsets - sample)) ? SAMPLES_IN_PACKET : impulsLenWithOffsets - sample;
				packetAssumptions[time].packetNumber = packetNumber++;
				packetAssumptions[time].impulsNumber = impuls;
			}
			
			//any other times
			else
			{
				packetAssumptions[time].canStartPacket = 0;
			}
			
			//time where is offset prev or post (the HW word is 0)
			if (sample < offsetPrev  || sample >= impulsLenWithOffsets - offsetPost)
			{
				packetAssumptions[time].isOffsetPrevPost = 1;
			}
			else
			{
				packetAssumptions[time].isOffsetPrevPost = 0;
			}
			
			time++;
		}
	}
	//fprintf(stderr, "B\n");
	packetsInSequence = packetNumber;
	sequenceLength = time;
}

void PrintPacketAssumptions(unsigned int offsetPrev, unsigned int offsetPost)
{
	PacketAssumptionsPrepare(offsetPrev, offsetPost);
	
	printf("Sequence length: real: %d, allocated array: %d\n", sequenceLength, sequenceLengthPrepared);
	printf("Packets in sequence: %d\n\n", packetsInSequence);
	
	int i;
	int prevInum = 0;
	int packetNumStart = 0;
	for (i = 0; i < sequenceLength; i++)
	{
		if (packetAssumptions[i].canStartPacket)
		{
			packetNumStart = i;
			
			if (packetAssumptions[i].impulsNumber != prevInum)
			{
				printf("\n");
				prevInum = packetAssumptions[i].impulsNumber;
			}
			
			printf("\ntime: %5d, %.2x, len: %3d, pNum: %3d, iNum: %3d, offset HW words: ", i, packetAssumptions[i].FP_LP, packetAssumptions[i].packetLength, packetAssumptions[i].packetNumber, packetAssumptions[i].impulsNumber);
		}
		
		if (packetAssumptions[i].isOffsetPrevPost)
		{
			printf("%d, ", i - packetNumStart);
		}
		
	}
	
	printf("\n");
}

static unsigned long long numWrongLP = 0;
static unsigned long long numWrongPacketLength = 0;
static unsigned long long packetTested = 0;

//static int writePacketPost = 0;

static unsigned int timeInSequencePrev;
static unsigned int packetNumPrev;
static unsigned int impulsNumPrev;
static unsigned long long timeStampPrev;
static unsigned char FP_LPPrev;
static unsigned int dataLenPrev;
static unsigned int expectedPacketNumPrev;
static unsigned int expectedImpulsNumPrev;
static unsigned int lastPacketNumWritten;
static unsigned long long packetTestedPrev;
static char dataPrev[6000];	//prepared for jumbo frame

void WritePacketContent(unsigned long long timeStamp, char* data, unsigned int dataLen)
{
	fprintf(stderr, "\n\nExpected firts time stamp in data: 0x%llx", timeStamp - DELAY_IN_EMBEDDED_DATA);
	
	int a;
	for (a = 0; a < dataLen; a++)
	{
		if (a%24 == 0)
		{
			fprintf(stderr, "\n%2d: ", a/24);
		}

		fprintf(stderr, "%2x", (unsigned char)data[a]);
	}
	
	fprintf(stderr, "\n---------------------------------------------------------------------------------\n\n\n");	
}

void WriteHeaderWithExpecting(unsigned long long timeStamp, unsigned int timeInSequence, unsigned long long packetTested, unsigned int expectedPacketNum, unsigned int packetNumber, unsigned int expectedImpulsNum,
								unsigned int impulsNumber, unsigned char expectedFP_LP, unsigned char FP_LP, unsigned int expectedPacketLength, unsigned int packetLength)
{
	fprintf(stderr, "\n\n---------------------------------------------------------------------------------\n");
	fprintf(stderr, "Time in sequence: %7u   Time: %llu (0x%llx)\n", timeInSequence, timeStamp, timeStamp);
	fprintf(stderr, "Packet arrived as number: %7llu\n", packetTested);
	fprintf(stderr, "----------------------------------------------------\n");
	fprintf(stderr, "                  expected         real\n");
	fprintf(stderr, "Packet number   %10u   %10u\n", expectedPacketNum, packetNumber);
	fprintf(stderr, "Impuls number   %10u   %10u\n", expectedImpulsNum, impulsNumber);
	fprintf(stderr, "FP_LP flag      %10.2x   %10.2x\n", expectedFP_LP, FP_LP);
	fprintf(stderr, "Packet length   %10u   %10u\n", expectedPacketLength, packetLength);
	fprintf(stderr, "----------------------------------------------------\n");
}

void WriteHeaderWithoutExpecting(unsigned int timeInSequence, unsigned long long timeStamp, unsigned int packetNumber, unsigned int impulsNumber, unsigned char FP_LP, unsigned int packetLength)
{
	fprintf(stderr, "\n\n---------------------------------------------------------------------------------\n");
	fprintf(stderr, "Time in sequence: %10u     Time: %15llu\n", timeInSequence, timeStamp);
	fprintf(stderr, "Packet number:    %10u\n", packetNumber);
	fprintf(stderr, "Impuls number:    %10u\n", impulsNumber);
	fprintf(stderr, "FP_LP:            %10.2x\n", FP_LP);
	fprintf(stderr, "Packet length:    %10u\n", packetLength);
	fprintf(stderr, "----------------------------------------------------\n");
}

static int writePacketHeader = 0;
static int writePacketHeaderWithExpecting = 0;

static int firstPacket = 1;
static unsigned long long packetShift = 0;


void TimeTestMode_TestPacket(unsigned int packetNumber, unsigned int impulsNumber, unsigned long long timeStamp, unsigned char FP_LP, char* data, unsigned int dataLen)
{
	//time in packets can be shifted
	//find delay between time packet assumption and realy value
	if (firstPacket == 1)
	{
		firstPacket = 0;
		
		for (packetShift = 0; packetShift <= 6; packetShift++)
		{
			if (packetAssumptions[timeStamp - packetShift].canStartPacket == 1)
			{
				 break;
			}
		}
		
		printf("Packet time shift: %llu\n", packetShift);
		
		if (packetShift == 6)
		{
			fprintf(stderr, "ERROR: Can not find time shift of first packet.\n");
			abort;
		}
	}
	
	
	timeStamp -= packetShift;
	
	
	///////////////////////////////////////////////////////////////////////////////
	///////////////////////////////////////////////////////////////////////////////
	// if (packetNumber % 1543 == 0 || packetNumber % 1543 == 1)
		// timeStamp++;
		
	// if (packetNumber % 1349 == 0)
		// packetNumber++;
		
	// if (packetNumber % 1983 == 0)
		// impulsNumber++;
		
	// if (packetNumber % 1763 == 0)
		// FP_LP = 176;
		
	// if (packetNumber % 1547 == 0)
		// dataLen = 66*24;
	///////////////////////////////////////////////////////////////////////////////
	///////////////////////////////////////////////////////////////////////////////
	
	packetTested++;
	
	unsigned int timeInSequence = (unsigned int)(timeStamp % (unsigned long long)sequenceLength);
	unsigned long long sequenceNumber = timeStamp / sequenceLength;
	unsigned int expectedPacketNum = (unsigned int)(sequenceNumber * packetsInSequence + (unsigned long long)packetAssumptions[timeInSequence].packetNumber);
	unsigned int expectedImpulsNum = (unsigned int)(sequenceNumber * IMPULSES_IN_SEQUENCE + (unsigned long long)packetAssumptions[timeInSequence].impulsNumber);
	
	if (packetAssumptions[timeInSequence].canStartPacket != 1)
	{
		fprintf(stderr, "\nERROR: At this time can not start any packet - packet content is NOT tested (packetNum %d)", packetNumber);	
		writePacketHeader = 2;
	}
	else
	{
		if (expectedImpulsNum != impulsNumber)
		{
			fprintf(stderr, "\nERROR: Wrong impuls number (packetNum %d)", packetNumber);	
			writePacketHeaderWithExpecting = 2;
		}
		
		if (expectedPacketNum != packetNumber)
		{
			fprintf(stderr, "\nERROR: Wrong packet number (packetNum %d)", packetNumber);	
			writePacketHeaderWithExpecting = 2;
		}
		
		
		if ((FP_LP&0xF0) != (packetAssumptions[timeInSequence].FP_LP&0xF0))
		{
			fprintf(stderr, "\nERROR: Wrong FP flag (packetNum %d)", packetNumber);	
			writePacketHeaderWithExpecting = 2;
		}
		
		
		if (dataLen / 24 > packetAssumptions[timeInSequence].packetLength)
		{
			fprintf(stderr, "\nERROR: Packet length is longer than expected (packetNum %d)", packetNumber);	
			writePacketHeaderWithExpecting = 2;
		}
		
		if (FP_LP&0x0F != packetAssumptions[timeInSequence].FP_LP&0x0F)
		{
			numWrongLP++;
		}
		
		if (dataLen / 24 != packetAssumptions[timeInSequence].packetLength)
		{
			numWrongPacketLength++;
		}
	}
	
	if (packetTested % 0x100000 == 0)
	{
		printf("\rTested packets: %13llu   Wrong LP flags:   Shorter packet lengths: ", packetTested);
		fflush(stdout);
	}
	
	//test packet contain
	if (packetAssumptions[timeInSequence].canStartPacket)
	{
		timeStampExpected.ull = timeStamp - DELAY_IN_EMBEDDED_DATA;
		
		int HWworldLength = 24;
		int writeWrongHWword = 0;
		int i;
		for (i = 0; i < dataLen / HWworldLength; i++)
		{
			int shift = (timeStampExpected.ull) % 24;
			int start = i*HWworldLength + (HWworldLength - 1) - shift;
			
			int writeWrongWordError = 0;
			int j;
			
			for (j = 0; j < HWworldLength; j++)
			{	
				int position = -j;
				if (position < -(start % 24)) 
				{
					position += 24;
				}
				
				int index = start + position;
				
				if (j < 8 && !(packetAssumptions[timeInSequence + i].isOffsetPrevPost))
				{
					if (data[index] != timeStampExpected.arr[j])
					{
						writeWrongWordError = 1;
					}
				}
				else
				{
					if (data[index] != 0)
					{
						writeWrongWordError = 1;
					}
				}	
			}
			
			if (writeWrongWordError)
			{
				fprintf(stderr, "\nERROR: in HW word %d, expected embeded time: 0x%llx (packetNum %d)", i, packetAssumptions[timeInSequence + i].isOffsetPrevPost ? 0 : timeStampExpected.ull, packetNumber);
				writePacketHeaderWithExpecting = 2;
			}
			
			timeStampExpected.ull++;	
		}
	}
	
	if (writePacketHeaderWithExpecting)
	{
		//write previous packet
		if (lastPacketNumWritten != packetNumPrev)
		{
			WriteHeaderWithExpecting(timeStampPrev, timeInSequencePrev, packetTestedPrev, expectedPacketNumPrev, packetNumPrev, expectedImpulsNumPrev,
									impulsNumPrev, packetAssumptions[timeInSequencePrev].FP_LP, FP_LPPrev, packetAssumptions[timeInSequencePrev].packetLength, dataLenPrev / 24);
		
			WritePacketContent(timeStampPrev, dataPrev, dataLenPrev);
		}
		
		WriteHeaderWithExpecting(timeStamp, timeInSequence, packetTested, expectedPacketNum, packetNumber, expectedImpulsNum,
								impulsNumber, packetAssumptions[timeInSequence].FP_LP, FP_LP, packetAssumptions[timeInSequence].packetLength, dataLen / 24);
		
		WritePacketContent(timeStamp, data, dataLen);
		
		lastPacketNumWritten = packetNumber;
	}
	
	if (writePacketHeader)
	{
		//write previous packet
		if (lastPacketNumWritten != packetNumPrev)
		{
			WriteHeaderWithoutExpecting(timeInSequencePrev, timeStampPrev, packetNumPrev, impulsNumPrev, FP_LPPrev, dataLenPrev / 24);
			WritePacketContent(timeStampPrev, dataPrev, dataLenPrev);
		}
		
		WriteHeaderWithoutExpecting(timeInSequence, timeStamp, packetNumber, impulsNumber, FP_LP, dataLen / 24);
		
		WritePacketContent(timeStamp, data, dataLen);
		
		lastPacketNumWritten = packetNumber;
	}
	
	if (writePacketHeaderWithExpecting)
	{
		writePacketHeaderWithExpecting--;
	}
	
	if (writePacketHeader)
	{
		writePacketHeader--;
	}
	
	timeInSequencePrev  = timeInSequence;
	packetNumPrev = packetNumber;
	impulsNumPrev = impulsNumber;
	timeStampPrev = timeStamp;
	FP_LPPrev = FP_LP;
	dataLenPrev = dataLen;
	expectedPacketNumPrev = expectedPacketNum;
	expectedImpulsNumPrev = expectedImpulsNum;
	packetTestedPrev = packetTested;
	memcpy(dataPrev, data, dataLen);
}

